using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using System.Windows.Media;


namespace MotionAPI_XNA_AR
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class CameraFeed : VideoSink
    {
        GraphicsDevice graphicsDevice;
        public Texture2D Texture;
        byte[] data;
        int pixelCount;
        int bytesPerPixel;

        public CameraFeed(GraphicsDevice graphicsDevice)
        {
            this.graphicsDevice = graphicsDevice;
            AllocationMode = SampleAllocationMode.ReuseBuffer;
        }

        protected override void OnCaptureStarted()
        {
        }

        protected override void OnCaptureStopped()
        {
        }

        protected override void OnFormatChange(VideoFormat videoFormat)
        {
            if (Texture != null)
                Texture.Dispose();
           
            Texture = new Texture2D(graphicsDevice, videoFormat.PixelWidth, videoFormat.PixelHeight);

            pixelCount = videoFormat.PixelWidth * videoFormat.PixelHeight;
            bytesPerPixel = 0;
            switch (videoFormat.PixelFormat)
            {
                case PixelFormatType.Format32bppArgb:
                    bytesPerPixel=4;                   
                    break;
                case PixelFormatType.Format8bppGrayscale:
                    bytesPerPixel = 1;
                    break;
            }

            data = new byte[pixelCount * 4];
                      
            for (int i = 0; i < pixelCount; i++)
            {
                data[i * 4 + 0] = 0;
                data[i * 4 + 1] = 0;
                data[i * 4 + 2] = 0;
                data[i * 4 + 3] = 255;
            }                   

            return;
        }

        protected override void OnSample(long sampleTimeInHundredNanoseconds, long frameDurationInHundredNanoseconds, byte[] sampleData)
        {
            if (Texture != null && bytesPerPixel==4)
            {
                for (int i = 0; i < pixelCount; i++ )
                {
                    byte b = sampleData[i * bytesPerPixel + 0];
                    byte g = sampleData[i * bytesPerPixel + 1];
                    byte r = sampleData[i * bytesPerPixel + 2];
                                        
                    data[i * 4 + 0] = r;
                    data[i * 4 + 1] = g;
                    data[i * 4 + 2] = b;
                }
                try { Texture.SetData(data); }
                catch (Exception e) { }
            } 
            if (Texture != null && bytesPerPixel == 1)
            {
                for (int i = 0; i < pixelCount; i++ )
                {   
                    byte gray = sampleData[i];
                    data[i * 4 + 0] = (byte)gray;
                    data[i * 4 + 1] = (byte)gray;
                    data[i * 4 + 2] = (byte)gray;
                }
                try {Texture.SetData(data);}
                catch(Exception e) {}
            }

            return;
        }


    }
}
