using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Devices.Sensors;
using System.Windows;


namespace MotionAPI_XNA_AR
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class GameComponent1 : Microsoft.Xna.Framework.DrawableGameComponent
    {
        SpriteBatch spriteBatch;
        Model model;
        Motion motion;

        Matrix world;
        Matrix view;
        Matrix projection;

        Vector3 cameraPosition;
        Vector3 shipPosition;

        //camera
        System.Windows.Media.CaptureSource captureSource;
        CameraFeed cameraFeed;

        public GameComponent1(Game game): base(game)
        {
            
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            spriteBatch = new SpriteBatch(this.Game.GraphicsDevice);

            //start camera feed
            captureSource = new System.Windows.Media.CaptureSource();
            captureSource.AudioCaptureDevice = null;
            captureSource.VideoCaptureDevice = System.Windows.Media.CaptureDeviceConfiguration.GetDefaultVideoCaptureDevice();
            captureSource.VideoCaptureDevice.DesiredFormat = new System.Windows.Media.VideoFormat(System.Windows.Media.PixelFormatType.Format8bppGrayscale, 640, 480, 30);
            cameraFeed = new CameraFeed(this.Game.GraphicsDevice);
            cameraFeed.CaptureSource = captureSource;
            captureSource.Start();

            //start motion sensor
            motion = new Motion();
            try
            {
                motion.TimeBetweenUpdates = TimeSpan.FromMilliseconds(16.666);
                motion.Start();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message,ex.GetType().Name,MessageBoxButton.OK);
                throw ex;
            }

            //world view proj
            cameraPosition = Vector3.Up * 1.0f;
            shipPosition = Vector3.Up * 1.0f + Vector3.Forward * 10;
            world = Matrix.CreateScale(1f / 800f)
                    * Matrix.CreateRotationY((float)Math.PI)
                    * Matrix.CreateRotationX((float)Math.PI/16f)
                    * Matrix.CreateTranslation(shipPosition);
            view = Matrix.CreateLookAt(cameraPosition, cameraPosition + Vector3.Forward, Vector3.Up);
            projection = Matrix.CreatePerspectiveFieldOfView((float)Math.PI / 4f, 800f / 480f, 0.100f, 100f);

            base.Initialize();
        }

        protected override void LoadContent()
        {
            model = this.Game.Content.Load<Model>("Ship");
            
            return;
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {            
            AttitudeReading attitude = motion.CurrentValue.Attitude;

            //corrent the rotation matrix from Motion api
            Matrix orientation = Matrix.Identity;
            orientation = Matrix.CreateRotationX(MathHelper.PiOver2); //device->screen cordinate fix
            orientation *= attitude.RotationMatrix;
            orientation *= Matrix.CreateRotationZ(MathHelper.PiOver2); //portrait->landscape cordinate fix

            //extract oriantation from view matrixs
            Vector3 cameraForward = Vector3.Zero;
            Vector3 cameraUp = Vector3.Zero;
            GetViewOrientation(ref orientation, out cameraForward, out cameraUp);
            
            view = Matrix.CreateLookAt(cameraPosition, cameraPosition + cameraForward, cameraUp);
            
            base.Update(gameTime);
        }

        public void GetViewOrientation(ref Matrix view, out Vector3 forward, out Vector3 up)
        {
            forward = new Vector3(-view.M13, -view.M23, -view.M33);
            up = new Vector3(view.M12, view.M22, view.M32);
            return;
        }

        public override void Draw(GameTime gameTime)
        {
            DrawCameraFeed();
            DrawShip();
        }

        private void DrawCameraFeed()
        {
            Texture2D texture = cameraFeed.Texture;
            if (texture == null) return;
            
            Color[] color = new Color[texture.Width*texture.Height];
            texture.GetData<Color>(color); 

            spriteBatch.Begin();
            spriteBatch.Draw(texture, this.Game.GraphicsDevice.Viewport.Bounds, null, Microsoft.Xna.Framework.Color.White, 0.0f, Vector2.Zero, SpriteEffects.None, 0.0f);
            spriteBatch.End();
            
            //clear texture from device. 
            Game.GraphicsDevice.Textures[0] = null;
            //GraphicsDevice.Clear(ClearOptions.DepthBuffer, Color.Black, , 0);

            return;
        }

        private void DrawShip()
        {
            GraphicsDevice.BlendState = BlendState.Opaque;
            GraphicsDevice.DepthStencilState = DepthStencilState.None;
            GraphicsDevice.RasterizerState = RasterizerState.CullNone;
            GraphicsDevice.SamplerStates[0] = SamplerState.LinearClamp;
                
            model.Draw(world, view, projection);

            return;
        }
    
    }
}
